﻿using ArtOfTest.WebAii.Core;
using ArtOfTest.WebAii.Wpf;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Threading;

namespace TelerikIssuePOC
{
    [TestClass]
    public class UnitTest1
    {
        private string exePath = @"C:\Temp\TestSortedView\TestSortedView.exe";
        private static string windowsProcessNameOfExe = "TestSortedView";
        private string logPath = @"C:\Temp\TelerikTestlog";
        [TestMethod]
        public void TestMethod1()
        {
            Start(exePath, logPath);
        }

        /// <summary>
        /// Gets the application.
        /// </summary>
        private static WpfApplication Application { get; set; }

        /// <summary>
        /// Gets the manager.
        /// </summary>
        private static Manager Manager { get; set; }

        /// <summary>
        /// Starts the test runner.
        /// </summary>
        public static void Start(string exePath, string logPath)
        {

            if (Manager == null)
            {
                StartUpManager(exePath, logPath);
            }

            try
            {
                // This is in case you have the application running when you start the test
                KillAllTestAppProcesses(windowsProcessNameOfExe);
                // Launch Application
                Application = Manager.LaunchNewApplication(Manager.Settings.Wpf.DefaultApplicationPath);
            }
            // sem to be getting a ReflectionType Load Exception 
            // LoaderExpection = "Could not load file or assembly 'Telerik.TestStudio.Interfaces, Version=2013.1.806.0, Culture=neutral, PublicKeyToken=b9f4c4e36181b3de' or one of its dependencies. The system cannot find the file specified.":"Telerik.TestStudio.Interfaces, Version=2013.1.806.0, Culture=neutral, PublicKeyToken=b9f4c4e36181b3de"
            // NB: This does not exist in Framework installation.
            catch(ReflectionTypeLoadException rtle)
            {
                Trace.WriteLine("************************");
                Trace.WriteLine("ReflectionTypeLoadException - " + rtle.Message);
                foreach (var le in rtle.LoaderExceptions)
                {
                    Trace.WriteLine(le.Message);
                }
                Trace.WriteLine("************************");

                // This is because even though the above throws an error the application still opens
                KillAllTestAppProcesses(windowsProcessNameOfExe);
                // NB: This time the exception above is not thrown when Application Launch is initiated
                Application = Manager.LaunchNewApplication(Manager.Settings.Wpf.DefaultApplicationPath);
            }
            Manager.ActiveApplication.WaitForWindow("MainWindow");

            //This line causes exactly the same error as when we start to Find controls
            try
            {
                Manager.ActiveApplication.MainWindow.RefreshVisualTrees();
            }
            catch(TypeInitializationException ex)
            {
                throw;
            }
        }

        /// <summary>
        /// The start up manager.
        /// </summary>
        /// <param name="atlasExePath">
        /// The exe path.
        /// </param>
        /// <param name="logPath">
        /// The log path.
        /// </param>
        private static void StartUpManager(string exePath, string logPath)
        {
            var settings = new Settings
            {
                ExecuteCommandTimeout = 30000,
                ExecutionDelay = 500,
                UnexpectedDialogAction = UnexpectedDialogAction.HandleAndFailTest,
                CreateLogFile = true,
                LogLocation = logPath,
                QueryEventLogErrorsOnExit = true,
                LogAnnotations = true,
                Wpf = new Settings.WpfSettings
                {
                    DefaultApplicationPath = exePath,
                },
                SimulatedMouseMoveSpeed = 1.0f
            };

            Manager = new Manager(settings);
            Manager.Start();
        }


        /// <summary>
        /// The kill all current exe processes.
        /// </summary>
        public static void KillAllTestAppProcesses(string processName)
        {
            var processes = Process.GetProcesses().Where(p => p.ProcessName == processName);
            if (processes.Any())
            {
                foreach (var atlasProcess in processes)
                {
                    if (!atlasProcess.WaitForExit(5000))
                    {
                        atlasProcess.Kill();
                    }
                }
            }
            // Give it a second for processes to be killed
            Thread.Sleep(1000);
        }

    }
}
